@@ -774,19 +774,18 @@ double Falign( int **whichmtx, double ***scoringmatrices, double **n_dynamicmtx,
 {
 	int i, j, k, l, m, maxk;
 	int nlen, nlen2, nlen4;
-	static TLS int prevalloclen = 0;
 	static TLS int crossscoresize = 0;
-	static TLS char **tmpseq1 = NULL;
-	static TLS char **tmpseq2 = NULL;
-	static TLS char **tmpptr1 = NULL;
-	static TLS char **tmpptr2 = NULL;
-	static TLS char **tmpres1 = NULL;
-	static TLS char **tmpres2 = NULL;
-	static TLS char **result1 = NULL;
-	static TLS char **result2 = NULL;
+	char **tmpseq1 = NULL;
+	char **tmpseq2 = NULL;
+	char **tmpptr1 = NULL;
+	char **tmpptr2 = NULL;
+	char **tmpres1 = NULL;
+	char **tmpres2 = NULL;
+	char **result1 = NULL;
+	char **result2 = NULL;
 #if RND
-	static TLS char **rndseq1 = NULL;
-	static TLS char **rndseq2 = NULL;
+	char **rndseq1 = NULL;
+	char **rndseq2 = NULL;
 #endif
 	static TLS Fukusosuu **seqVector1 = NULL;
 	static TLS Fukusosuu **seqVector2 = NULL;
@@ -803,7 +802,7 @@ double Falign( int **whichmtx, double ***scoringmatrices, double **n_dynamicmtx,
 	static TLS Segment **sortedseg2 = NULL;
 	static TLS int *cut1 = NULL;
 	static TLS int *cut2 = NULL;
-	static TLS char *sgap1, *egap1, *sgap2, *egap2;
+	char *sgap1, *egap1, *sgap2, *egap2;
 	static TLS int localalloclen = 0;
 	int lag;
 	int tmpint;
@@ -817,11 +816,10 @@ double Falign( int **whichmtx, double ***scoringmatrices, double **n_dynamicmtx,
 
 	if( seq1 == NULL )
 	{
-		if( result1 ) 
+		if( kouho ) 
 		{
 //			fprintf( stderr, "Freeing localarrays in Falign\n" );
 			localalloclen = 0;
-			prevalloclen = 0;
 			crossscoresize = 0;
 			mymergesort( 0, 0, NULL );
 			alignableReagion( 0, 0, NULL, NULL, NULL, NULL, NULL );
@@ -834,21 +832,10 @@ double Falign( int **whichmtx, double ***scoringmatrices, double **n_dynamicmtx,
 			blockAlign2( NULL, NULL, NULL, NULL, NULL, NULL );
 			if( crossscore ) FreeDoubleMtx( crossscore );
 			crossscore = NULL;
-			FreeCharMtx( result1 ); result1 = NULL;
-			FreeCharMtx( result2 );
-			FreeCharMtx( tmpres1 );
-			FreeCharMtx( tmpres2 );
-			FreeCharMtx( tmpseq1 );
-			FreeCharMtx( tmpseq2 );
-			free( sgap1 );
-			free( egap1 );
-			free( sgap2 );
-			free( egap2 );
 			free( kouho );
+			kouho = NULL;
 			free( cut1 );
 			free( cut2 );
-			free( tmpptr1 );
-			free( tmpptr2 );
 			free( segment );
 			free( segment1 );
 			free( segment2 );
@@ -888,33 +875,33 @@ double Falign( int **whichmtx, double ***scoringmatrices, double **n_dynamicmtx,
 	fprintf( stderr, "nlentmp = %d, nlen = %d\n", nlentmp, nlen );
 #endif
 
-	if( prevalloclen != alloclen ) // Falign_noudp mo kaeru
-	{
-		if( prevalloclen )
-		{
-			FreeCharMtx( result1 );
-			FreeCharMtx( result2 );
-			FreeCharMtx( tmpres1 );
-			FreeCharMtx( tmpres2 );
-		}
-//		fprintf( stderr, "\n\n\nreallocating ...\n" ); 
-		result1 = AllocateCharMtx( njob, alloclen );
-		result2 = AllocateCharMtx( njob, alloclen );
-		tmpres1 = AllocateCharMtx( njob, alloclen );
-		tmpres2 = AllocateCharMtx( njob, alloclen );
-		prevalloclen = alloclen;
-	}
+	result1 = AllocateCharMtx( clus1, alloclen );
+	result2 = AllocateCharMtx( clus2, alloclen );
+	tmpres1 = AllocateCharMtx( clus1, alloclen );
+	tmpres2 = AllocateCharMtx( clus2, alloclen );
+	sgap1 = AllocateCharVec( clus1 );
+	egap1 = AllocateCharVec( clus1 );
+	sgap2 = AllocateCharVec( clus2 );
+	egap2 = AllocateCharVec( clus2 );
+	tmpptr1 = calloc( clus1, sizeof( char * ) );
+	tmpptr2 = calloc( clus2, sizeof( char * ) );
+	tmpseq1 = AllocateCharMtx( clus1, nlen );
+	tmpseq2 = AllocateCharMtx( clus2, nlen );
+#if RND
+	rndseq1 = AllocateCharMtx( clus1, nlen );
+	rndseq2 = AllocateCharMtx( clus2, nlen );
+	for( i=0; i<clus1; i++ )
+		generateRndSeq( rndseq1[i], nlen );
+	for( i=0; i<clus2; i++ )
+		generateRndSeq( rndseq2[i], nlen );
+#endif
+
+
 	if( !localalloclen )
 	{
-		sgap1 = AllocateCharVec( njob );
-		egap1 = AllocateCharVec( njob );
-		sgap2 = AllocateCharVec( njob );
-		egap2 = AllocateCharVec( njob );
 		kouho = AllocateIntVec( NKOUHO );
 		cut1 = AllocateIntVec( MAXSEG );
 		cut2 = AllocateIntVec( MAXSEG );
-		tmpptr1 = AllocateCharMtx( njob, 0 );
-		tmpptr2 = AllocateCharMtx( njob, 0 );
 //		crossscore = AllocateDoubleMtx( MAXSEG, MAXSEG );
 		segment = (Segment *)calloc( MAXSEG, sizeof( Segment ) );
 		segment1 = (Segment *)calloc( MAXSEG, sizeof( Segment ) );
@@ -941,17 +928,9 @@ double Falign( int **whichmtx, double ***scoringmatrices, double **n_dynamicmtx,
 				FreeFukusosuuMtx( naiseki );
 				FreeDoubleVec( soukan );
 			}
-			FreeCharMtx( tmpseq1 );
-			FreeCharMtx( tmpseq2 );
-#endif
-#if RND
-			FreeCharMtx( rndseq1 );
-			FreeCharMtx( rndseq2 );
 #endif
 		}
 
-		tmpseq1 = AllocateCharMtx( njob, nlen );
-		tmpseq2 = AllocateCharMtx( njob, nlen );
 		if( !kobetsubunkatsu )
 		{
 			naisekiNoWa = AllocateFukusosuuVec( nlen );
@@ -960,18 +939,9 @@ double Falign( int **whichmtx, double ***scoringmatrices, double **n_dynamicmtx,
 			seqVector2 = AllocateFukusosuuMtx( n20or4or2+1, nlen+1 );
 			soukan = AllocateDoubleVec( nlen+1 );
 		}
-#if RND
-		rndseq1 = AllocateCharMtx( njob, nlen );
-		rndseq2 = AllocateCharMtx( njob, nlen );
-		for( i=0; i<njob; i++ )
-		{
-			generateRndSeq( rndseq1[i], nlen );
-			generateRndSeq( rndseq2[i], nlen );
-		}
-#endif
 		localalloclen = nlen;
 	}
-	
+
 	for( j=0; j<clus1; j++ ) strcpy( tmpseq1[j], seq1[j] );
 	for( j=0; j<clus2; j++ ) strcpy( tmpseq2[j], seq2[j] );
 
@@ -1610,6 +1580,7 @@ system( "less seqVec2 < /dev/tty > /dev/tty" );
 		}
 #endif
 	}
+
 #if KEIKA
 	fprintf( stderr, "DP ... done   \n" );
 #endif
@@ -1627,6 +1598,24 @@ system( "less seqVec2 < /dev/tty > /dev/tty" );
 		fprintf( stderr, "in Falign, %s\n", result2[j] );
 	}
 #endif
+
+	FreeCharMtx( result1 );
+	FreeCharMtx( result2 );
+	FreeCharMtx( tmpres1 );
+	FreeCharMtx( tmpres2 );
+	FreeCharMtx( tmpseq1 );
+	FreeCharMtx( tmpseq2 );
+	free( sgap1 );
+	free( egap1 );
+	free( sgap2 );
+	free( egap2 );
+	free( tmpptr1 );
+	free( tmpptr2 );
+#if RND
+	FreeCharMtx( rndseq1 );
+	FreeCharMtx( rndseq2 );
+#endif
+
 	return( totalscore );
 }
 
@@ -1671,6 +1660,7 @@ static int terminalmargin( int lshorter, double groupsizefac )
 //	return ( lshorter * 1.1 + 10 ) * groupsizefac;
 }
 
+#if 0
 static int estimatenogaplen( int n, char **s, int start, int end )
 {
 	int i, j, l, d, o;
@@ -1732,6 +1722,7 @@ static int nogapmargin( int n, char **s, int start, int end, int m )
 //	reporterr( "minl=%d, so returning %d\n", minl, start+minl*d );
 	return( start + minl*d );
 }
+#endif
 
 double Falign_givenanchors( ExtAnch *pairanch, 
 			  int **whichmtx, double ***scoringmatrices, 
@@ -1876,7 +1867,7 @@ double Falign_givenanchors( ExtAnch *pairanch,
 			FreeCharMtx( tmpres2 );
 		}
 //		fprintf( stderr, "\n\n\nreallocating ...\n" ); 
-		result1 = AllocateCharMtx( njob, alloclen );
+		result1 = AllocateCharMtx( njob, alloclen ); // ato de loca nseq ni kakihaosu
 		result2 = AllocateCharMtx( njob, alloclen );
 		tmpres1 = AllocateCharMtx( njob, alloclen );
 		tmpres2 = AllocateCharMtx( njob, alloclen );
@@ -2388,8 +2379,9 @@ double Falign_givenanchors( ExtAnch *pairanch,
 		cut1[count+1] = pairanch[count0].endi+1;
 		cut2[count+1] = pairanch[count0].endj+1;
 		alignorcopy[count+1] = 'a';
-//		reporterr( "\n###cut at %d and %d / %d\n", cut1[count+1], cut1[count+2], len1 );
-//		reporterr( "\n###cut at %d and %d / %d\n", cut2[count+1], cut2[count+2], len2 );
+//		reporterr( "\n###cut1 at %d / %d\n", cut1[count+1], len1 );
+//		reporterr( "###cut2 at %d / %d\n", cut2[count+1], len2 );
+//		reporterr( "sa1=%d, sa2=%d\n", cut1[count+1]-cut1[count], cut2[count+1]-cut2[count] );
 		count += 1;
 		count0++;
 	}
@@ -2455,6 +2447,7 @@ double Falign_givenanchors( ExtAnch *pairanch,
 	count += 1;
 
 
+
 #if 0
 	for( i=0; i<count; i++ )
 	{
@@ -2477,6 +2470,7 @@ double Falign_givenanchors( ExtAnch *pairanch,
 	count += 2;
 #endif
 
+
 	totallen = 0;
 	for( j=0; j<clus1; j++ ) result1[j][0] = 0;
 	for( j=0; j<clus2; j++ ) result2[j][0] = 0;
@@ -2492,8 +2486,8 @@ double Falign_givenanchors( ExtAnch *pairanch,
 		//reporterr( "i=%d, headgp=%d\n", i, headgp );
 		//reporterr( "i=%d, tailgp=%d\n", i, tailgp );
 
-		//for( j=0; j<clus1; j++ ) reporterr( "Cut seq 0-%d at %d\n", j, cut1[i] );
-		//for( j=0; j<clus2; j++ ) reporterr( "Cut seq 1-%d at %d\n", j, cut2[i] );
+//		for( j=0; j<clus1; j++ ) reporterr( "Cut seq 0-%d at %d\n", j, cut1[i] );
+//		for( j=0; j<clus2; j++ ) reporterr( "Cut seq 1-%d at %d\n", j, cut2[i] );
 
 #if 0 // hoka mo atode henkou suru
 		if( cut1[i] )
@@ -2684,13 +2678,13 @@ double Falign_givenanchors( ExtAnch *pairanch,
 			int tmplen;
 			if( endtermcut1 )
 			{
-				tmplen = strlen( tmpres2[j] );
+				tmplen = strlen( tmpres2[0] );
 				for( j=0; j<clus2; j++ ) if( tmpres2[j][tmplen-1] != '-' ) break;
 				if( j<clus2 ) reporterr( "There may be a problem in a hard-coded parameter (3).  Please contact katoh@ifrec.osaka-u.ac.jp\n" );
 			}
 			else if( endtermcut2 )
 			{
-				tmplen = strlen( tmpres1[j] );
+				tmplen = strlen( tmpres1[0] );
 				for( j=0; j<clus1; j++ ) if( tmpres1[j][tmplen-1] != '-' ) break;
 				if( j<clus1 ) reporterr( "There may be a problem in a hard-coded parameter (4).  Please contact katoh@ifrec.osaka-u.ac.jp\n" );
 			}
@@ -2733,14 +2727,14 @@ double Falign_givenanchors( ExtAnch *pairanch,
 	fprintf( stderr, "keika \n\n" );
 	for( j=0; j<clus1; j++ ) 
 	{
-		fprintf( stderr, ">group1-%d\n%s\n", j, result1[j] );
+		fprintf( stderr, ">group1-%d\n%100.100s\n", j, result1[j] );
 	}
 	fprintf( stderr, "- - - - - - - - - - -\n" );
 	for( j=0; j<clus2; j++ ) 
 	{
-		fprintf( stderr, ">group2-%d\n%s\n", j, result2[j] );
+		fprintf( stderr, ">group2-%d\n%100.100s\n", j, result2[j] );
 	}
-	if( clus1 == 2 ) exit( 1 );
+//	if( clus1 == 1 && clus2 == 5 ) exit( 1 );
 #endif
 	return( totalscore );
 }
@@ -2752,7 +2746,7 @@ double Falign_givenanchors( ExtAnch *pairanch,
 /*
 sakujo wo kentou (2010/10/05)
 */
-double Falign_udpari_long( 
+double Falign_udpari_long(
 			  int **whichmtx, double ***scoringmatrices, 
 			  double **n_dynamicmtx,
 			  char  **seq1, char  **seq2, 
@@ -2763,19 +2757,18 @@ double Falign_udpari_long(
 {
 	int i, j, k, l, m, maxk;
 	int nlen, nlen2, nlen4;
-	static TLS int prevalloclen = 0;
 	static TLS int crossscoresize = 0;
-	static TLS char **tmpseq1 = NULL;
-	static TLS char **tmpseq2 = NULL;
-	static TLS char **tmpptr1 = NULL;
-	static TLS char **tmpptr2 = NULL;
-	static TLS char **tmpres1 = NULL;
-	static TLS char **tmpres2 = NULL;
-	static TLS char **result1 = NULL;
-	static TLS char **result2 = NULL;
+	char **tmpseq1 = NULL;
+	char **tmpseq2 = NULL;
+	char **tmpptr1 = NULL;
+	char **tmpptr2 = NULL;
+	char **tmpres1 = NULL;
+	char **tmpres2 = NULL;
+	char **result1 = NULL;
+	char **result2 = NULL;
 #if RND
-	static TLS char **rndseq1 = NULL;
-	static TLS char **rndseq2 = NULL;
+	char **rndseq1 = NULL;
+	char **rndseq2 = NULL;
 #endif
 	static TLS Fukusosuu **seqVector1 = NULL;
 	static TLS Fukusosuu **seqVector2 = NULL;
@@ -2792,7 +2785,7 @@ double Falign_udpari_long(
 	static TLS Segment **sortedseg2 = NULL;
 	static TLS int *cut1 = NULL;
 	static TLS int *cut2 = NULL;
-	static TLS char *sgap1, *egap1, *sgap2, *egap2;
+	char *sgap1, *egap1, *sgap2, *egap2;
 	static TLS int localalloclen = 0;
 	int lag;
 	int tmpint;
@@ -2806,11 +2799,10 @@ double Falign_udpari_long(
 
 	if( seq1 == NULL )
 	{
-		if( result1 ) 
+		if( kouho ) 
 		{
 //			fprintf( stderr, "### Freeing localarrays in Falign\n" );
 			localalloclen = 0;
-			prevalloclen = 0;
 			crossscoresize = 0;
 			mymergesort( 0, 0, NULL );
 			alignableReagion( 0, 0, NULL, NULL, NULL, NULL, NULL );
@@ -2822,21 +2814,10 @@ double Falign_udpari_long(
 			blockAlign2( NULL, NULL, NULL, NULL, NULL, NULL );
 			if( crossscore ) FreeDoubleMtx( crossscore );
 			crossscore = NULL; // reallocate sareru kanousei ga arunode.
-			FreeCharMtx( result1 ); result1 = NULL;
-			FreeCharMtx( result2 );
-			FreeCharMtx( tmpres1 );
-			FreeCharMtx( tmpres2 );
-			FreeCharMtx( tmpseq1 );
-			FreeCharMtx( tmpseq2 );
-			free( sgap1 );
-			free( egap1 );
-			free( sgap2 );
-			free( egap2 );
 			free( kouho );
+			kouho = NULL;
 			free( cut1 );
 			free( cut2 );
-			free( tmpptr1 );
-			free( tmpptr2 );
 			free( segment );
 			free( segment1 );
 			free( segment2 );
@@ -2859,8 +2840,6 @@ double Falign_udpari_long(
 		return( 0.0 );
 	}
 
-
-
 	len1 = strlen( seq1[0] );
 	len2 = strlen( seq2[0] );
 	nlentmp = MAX( len1, len2 );
@@ -2878,34 +2857,34 @@ double Falign_udpari_long(
 	fprintf( stderr, "nlentmp = %d, nlen = %d\n", nlentmp, nlen );
 #endif
 
-	if( prevalloclen != alloclen ) // Falign_noudp mo kaeru
-	{
-		if( prevalloclen )
-		{
-			FreeCharMtx( result1 );
-			FreeCharMtx( result2 );
-			FreeCharMtx( tmpres1 );
-			FreeCharMtx( tmpres2 );
-		}
-//		fprintf( stderr, "\n\n\nreallocating ...\n" ); 
-		result1 = AllocateCharMtx( njob, alloclen );
-		result2 = AllocateCharMtx( njob, alloclen );
-		tmpres1 = AllocateCharMtx( njob, alloclen );
-		tmpres2 = AllocateCharMtx( njob, alloclen );
-		prevalloclen = alloclen;
-	}
+	result1 = AllocateCharMtx( clus1, alloclen );
+	result2 = AllocateCharMtx( clus2, alloclen );
+	tmpres1 = AllocateCharMtx( clus1, alloclen );
+	tmpres2 = AllocateCharMtx( clus2, alloclen );
+	sgap1 = AllocateCharVec( clus1 );
+	egap1 = AllocateCharVec( clus1 );
+	sgap2 = AllocateCharVec( clus2 );
+	egap2 = AllocateCharVec( clus2 );
+
+	tmpseq1 = AllocateCharMtx( clus1, nlen );
+	tmpseq2 = AllocateCharMtx( clus2, nlen );
+	tmpptr1 = calloc( clus1, sizeof(char*) );
+	tmpptr2 = calloc( clus2, sizeof(char*) );
+
+#if RND
+	rndseq1 = AllocateCharMtx( clus1, nlen );
+	rndseq2 = AllocateCharMtx( clus2, nlen );
+	for( i=0; i<clus1; i++ )
+		generateRndSeq( rndseq1[i], nlen );
+	for( i=0; i<clus2; i++ )
+		generateRndSeq( rndseq2[i], nlen );
+#endif
 
 	if( !localalloclen )
 	{
-		sgap1 = AllocateCharVec( njob );
-		egap1 = AllocateCharVec( njob );
-		sgap2 = AllocateCharVec( njob );
-		egap2 = AllocateCharVec( njob );
 		kouho = AllocateIntVec( NKOUHO_LONG );
 		cut1 = AllocateIntVec( MAXSEG );
 		cut2 = AllocateIntVec( MAXSEG );
-		tmpptr1 = AllocateCharMtx( njob, 0 );
-		tmpptr2 = AllocateCharMtx( njob, 0 );
 		segment = (Segment *)calloc( MAXSEG, sizeof( Segment ) );
 		segment1 = (Segment *)calloc( MAXSEG, sizeof( Segment ) );
 		segment2 = (Segment *)calloc( MAXSEG, sizeof( Segment ) );
@@ -2918,6 +2897,7 @@ double Falign_udpari_long(
 		else if( fftscore )       n20or4or2 = 1;
 		else                      n20or4or2 = 20;
 	}
+
 	if( localalloclen < nlen )
 	{
 		if( localalloclen )
@@ -2931,18 +2911,10 @@ double Falign_udpari_long(
 				FreeFukusosuuMtx( naiseki );
 				FreeDoubleVec( soukan );
 			}
-			FreeCharMtx( tmpseq1 );
-			FreeCharMtx( tmpseq2 );
-#endif
-#if RND
-			FreeCharMtx( rndseq1 );
-			FreeCharMtx( rndseq2 );
 #endif
 		}
 
 
-		tmpseq1 = AllocateCharMtx( njob, nlen );
-		tmpseq2 = AllocateCharMtx( njob, nlen );
 		if( !kobetsubunkatsu )
 		{
 			naisekiNoWa = AllocateFukusosuuVec( nlen );
@@ -2951,15 +2923,6 @@ double Falign_udpari_long(
 			seqVector2 = AllocateFukusosuuMtx( n20or4or2, nlen+1 );
 			soukan = AllocateDoubleVec( nlen+1 );
 		}
-#if RND
-		rndseq1 = AllocateCharMtx( njob, nlen );
-		rndseq2 = AllocateCharMtx( njob, nlen );
-		for( i=0; i<njob; i++ )
-		{
-			generateRndSeq( rndseq1[i], nlen );
-			generateRndSeq( rndseq2[i], nlen );
-		}
-#endif
 		localalloclen = nlen;
 	}
 	
@@ -3554,6 +3517,7 @@ system( "less seqVec2 < /dev/tty > /dev/tty" );
 						totalscore += MSalignmm_variousdist( NULL, scoringmatrices, NULL, tmpres1, tmpres2, eff1, eff2, eff1s, eff2s, clus1, clus2, alloclen, sgap1, sgap2, egap1, egap2, NULL, 0, NULL, headgp, tailgp );
 					else
 						totalscore += MSalignmm( n_dynamicmtx, tmpres1, tmpres2, eff1, eff2, clus1, clus2, alloclen, sgap1, sgap2, egap1, egap2, NULL, 0, NULL, headgp, tailgp, NULL, NULL, NULL, 0.0, 0.0 );
+//						totalscore += G__align11( n_dynamicmtx, tmpres1, tmpres2, alloclen, headgp, tailgp ); // CHUUI!!!
 				break;
 			default:
 				fprintf( stderr, "alg = %c\n", alg );
@@ -3602,5 +3566,22 @@ system( "less seqVec2 < /dev/tty > /dev/tty" );
 		fprintf( stderr, "%s\n", result2[j] );
 	}
 #endif
+
+	FreeCharMtx( result1 );
+	FreeCharMtx( result2 );
+	FreeCharMtx( tmpres1 );
+	FreeCharMtx( tmpres2 );
+	free( sgap1 );
+	free( egap1 );
+	free( sgap2 );
+	free( egap2 );
+	FreeCharMtx( tmpseq1 );
+	FreeCharMtx( tmpseq2 );
+	free( tmpptr1 );
+	free( tmpptr2 );
+#if RND
+	FreeCharMtx( rndseq1 );
+	FreeCharMtx( rndseq2 );
+#endif
 	return( totalscore );
 }
